<?php

namespace App\Http\Controllers\Master\AccountMaster;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
// use App\Supplier;
use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use App\AllCountries;
use App\AccountMasterTrack;
use App\ClassOfAccount;
use App\User;
use App\Location;
use App\LocationPermission;
use Illuminate\Support\Facades\Hash;

class SupplierController extends Controller
{

    protected $utils;

    public function __construct(Utils $utils)
    {
        $this->utils = $utils;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('suppliers-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $suppliers = AccountMaster::Where('location_id', $location_id)->Where('account_master_type', "Supplier")->orderBy('created_at','DESC')->paginate(100);
        $count = [];
        foreach ($suppliers as $supplier) {
            $account_detail = AccountMastersAccountsDetail::Where('account_master_id', $supplier->id)->where('voucher_type', '!=' ,"opening balance")->get();
            $count[] = count($account_detail);
        }
        return view('user.masters.account_masters.suppliers.index', compact('suppliers', 'count'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('suppliers-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $country = Location::where('id', $location_id)->first();
        $class_of_accounts = ClassOfAccount::all();
        $users = LocationPermission::with('users')->where('country_id', $country->country_id)->where('location_id', $location_id)->where('user_id', '!=', $user->id)->get();
        $all_countries = AllCountries::all();
        return view('user.masters.account_masters.suppliers.create', compact('all_countries', 'class_of_accounts', 'users'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $location_id = $this->getLocationId();
        $supplier = AccountMaster::Where('name_of_organisation', $request->name_of_organisation)->Where('account_master_type', "Supplier")->where('location_id', $location_id)->first();
        if ($supplier) {
            Session::flash('alert-danger', 'This Supplier Already Exists');
            return redirect()->back();
        }
        else
        $supplier = new AccountMaster();
        $supplier->location_id = $location_id;
        $supplier->account_master_type = "Supplier";
        if ($request->transactions_password != "") {
            $supplier->transactions_password = Hash::make($request->transactions_password);
        }
        $supplier->opening_balance = $request->opening_balance;
        $supplier->debit_credit = $request->debit_credit;
        if ($request->report_password != "") {
            $supplier->report_password = Hash::make($request->transactions_password);
        }
        $supplier->date = $request->date;
        $supplier->name_of_organisation = $request->name_of_organisation;
        $supplier->commerical_registration_number = $request->commerical_registration_number;
        $supplier->contact_person_name = $request->contact_person_name;
        $supplier->office_phone = $request->office_phone;
        $supplier->mobile = $request->mobile;
        $supplier->whatsapp = $request->whatsapp;
        $supplier->email = $request->email;
        $supplier->country_id = $request->country_id;
        $supplier->city = $request->city;
        $supplier->address = $request->address;
        $supplier->class_of_account_id = $request->class_of_account_id;
        // $supplier->show_user = $request->show_user;
        $supplier->maintain_account = $request->maintain_account;
        $supplier->vat_number = $request->vat_number;
        $supplier->trn_number = $request->trn_number;
        $supplier->tin_number = $request->tin_number;
        $supplier->doc_name_1 = $request->doc_name_1;
        if ($request->hasFile('doc_file_1')) {
            $file = $request->file('doc_file_1');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_1 = $filePath;
        }
        $supplier->doc_expiry_1 = $request->doc_expiry_1;

        $supplier->doc_name_2 = $request->doc_name_2;
        if ($request->hasFile('doc_file_2')) {
            $file = $request->file('doc_file_2');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_2 = $filePath;
        }
        $supplier->doc_expiry_2 = $request->doc_expiry_2;

        $supplier->doc_name_3 = $request->doc_name_3;
        if ($request->hasFile('doc_file_3')) {
            $file = $request->file('doc_file_3');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_3 = $filePath;
        }
        $supplier->doc_expiry_3 = $request->doc_expiry_3;

        $supplier->doc_name_4 = $request->doc_name_4;
        if ($request->hasFile('doc_file_4')) {
            $file = $request->file('doc_file_4');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_4 = $filePath;
        }
        $supplier->doc_expiry_4 = $request->doc_expiry_4;

        $supplier->doc_name_5 = $request->doc_name_5;
        if ($request->hasFile('doc_file_5')) {
            $file = $request->file('doc_file_5');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_5 = $filePath;
        }
        $supplier->doc_expiry_5 = $request->doc_expiry_5;

        $supplier->doc_name_6 = $request->doc_name_6;
        if ($request->hasFile('doc_file_6')) {
            $file = $request->file('doc_file_6');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_6 = $filePath;
        }
        $supplier->doc_expiry_6 = $request->doc_expiry_6;


        $supplier->doc_name_7 = $request->doc_name_7;
        if ($request->hasFile('doc_file_7')) {
            $file = $request->file('doc_file_7');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_7 = $filePath;
        }
        $supplier->doc_expiry_7 = $request->doc_expiry_7;

        $supplier->description = $request->description;
        $supplier->save();
        $current_supplier_id = $supplier->id;

        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $request->show_user;
                $account_master_track->account_master_id = $current_supplier_id;
                $account_master_track->account_master_name = "Supplier";
                $account_master_track->save();
            }
        }

        if(Auth::user()->hasRole('super-admin'))
        {
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $current_supplier_id;
            $account_master_track->account_master_name = "Supplier";
            $account_master_track->save();
        }else{
            $admins = User::role('super-admin')->get();
            foreach ($admins as $key => $admin) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $admin->id;
                $account_master_track->account_master_id = $current_supplier_id;
                $account_master_track->account_master_name = "Supplier";
                $account_master_track->save();
            }
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $current_supplier_id;
            $account_master_track->account_master_name = "Supplier";
            $account_master_track->save();
        }

        if ($request->opening_balance != "") {
            $account = new AccountMastersAccountsDetail;
            $account->account_master_id = $supplier->id;
            if ($request->debit_credit == "debit") {
                $account->debit = $request->opening_balance;
                $account->credit = 0;
            }elseif ($request->debit_credit == "credit") {
                $account->debit = 0;
                $account->credit = $request->opening_balance;
            }
            $account->date = $request->date;
            $account->voucher_type = "opening balance";
            $account->save();
        }


        Session::flash('alert-success', 'Supplier created successfully');
        return redirect('/masters/account-masters/suppliers');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Supplier  $supplier
     * @return \Illuminate\Http\Response
     */
    public function show(Supplier $supplier)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Supplier  $supplier
     * @return \Illuminate\Http\Response
     */
    public function edit($supplier_id)
    {
        $user = Auth::user();
        if (!$user->can('suppliers-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $country = Location::where('id', $location_id)->first();
        $class_of_accounts = ClassOfAccount::all();
        $users = LocationPermission::with('users')->where('country_id', $country->country_id)->where('location_id', $location_id)->where('user_id', '!=', $user->id)->get();
        $account_master_track = AccountMasterTrack::Where('account_master_id', $supplier_id)
                                                    ->Where('account_master_name', "Supplier")->pluck('user_id');
        $all_countries = AllCountries::all();
        $supplier = AccountMaster::Where('id', $supplier_id)->Where('account_master_type', "Supplier")->first();
        return view('user.masters.account_masters.suppliers.edit', compact('supplier', 'all_countries', 'class_of_accounts', 'users', 'account_master_track'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Supplier  $supplier
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $supplier_id)
    {

        $supplier = AccountMaster::Where('id', $supplier_id)->Where('account_master_type', "Supplier")->first();
        $previous_opening_balance = (float)$supplier->opening_balance;
        $current_opening_balance = (float)$request->opening_balance;
        if ($request->transactions_password != "") {
            $supplier->transactions_password = Hash::make($request->transactions_password) ? Hash::make($request->transactions_password) : $supplier->transactions_password;
        }
        $supplier->opening_balance = $request->opening_balance;
        if ($request->report_password != "") {
            $supplier->report_password = Hash::make($request->report_password) ? Hash::make($request->report_password) : $supplier->report_password;
        }
        $supplier->date = $request->date;
        $supplier->name_of_organisation = $request->name_of_organisation;
        $supplier->commerical_registration_number = $request->commerical_registration_number;
        $supplier->contact_person_name = $request->contact_person_name;
        $supplier->office_phone = $request->office_phone;
        $supplier->mobile = $request->mobile;
        $supplier->whatsapp = $request->whatsapp;
        $supplier->email = $request->email;
        $supplier->country_id = $request->country_id;
        $supplier->city = $request->city;
        $supplier->address = $request->address;
        $supplier->class_of_account_id = $request->class_of_account_id;
        // $supplier->show_user = $request->show_user;
        $supplier->maintain_account = $request->maintain_account;
        $supplier->vat_number = $request->vat_number;
        $supplier->trn_number = $request->trn_number;
        $supplier->tin_number = $request->tin_number;
        $supplier->doc_name_1 = $request->doc_name_1;
        if ($request->hasFile('doc_file_1')) {
            $file = $request->file('doc_file_1');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_1 = $filePath;
        }
        $supplier->doc_expiry_1 = $request->doc_expiry_1;

        $supplier->doc_name_2 = $request->doc_name_2;
        if ($request->hasFile('doc_file_2')) {
            $file = $request->file('doc_file_2');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_2 = $filePath;
        }
        $supplier->doc_expiry_2 = $request->doc_expiry_2;

        $supplier->doc_name_3 = $request->doc_name_3;
        if ($request->hasFile('doc_file_3')) {
            $file = $request->file('doc_file_3');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_3 = $filePath;
        }
        $supplier->doc_expiry_3 = $request->doc_expiry_3;

        $supplier->doc_name_4 = $request->doc_name_4;
        if ($request->hasFile('doc_file_4')) {
            $file = $request->file('doc_file_4');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_4 = $filePath;
        }
        $supplier->doc_expiry_4 = $request->doc_expiry_4;

        $supplier->doc_name_5 = $request->doc_name_5;
        if ($request->hasFile('doc_file_5')) {
            $file = $request->file('doc_file_5');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_5 = $filePath;
        }
        $supplier->doc_expiry_5 = $request->doc_expiry_5;

        $supplier->doc_name_6 = $request->doc_name_6;
        if ($request->hasFile('doc_file_6')) {
            $file = $request->file('doc_file_6');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_6 = $filePath;
        }
        $supplier->doc_expiry_6 = $request->doc_expiry_6;


        $supplier->doc_name_7 = $request->doc_name_7;
        if ($request->hasFile('doc_file_7')) {
            $file = $request->file('doc_file_7');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/supplier_doc/'), $filePath);
            $supplier-> doc_file_7 = $filePath;
        }
        $supplier->doc_expiry_7 = $request->doc_expiry_7;

        $supplier->description = $request->description;

        $supplier->save();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $supplier_id)
                                                    ->Where('account_master_name', "Supplier")->get();
        foreach ($account_master_track as $item) {
            $item->delete();
        }
        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $request->show_user;
                $account_master_track->account_master_id = $supplier_id;
                $account_master_track->account_master_name = "Supplier";
                $account_master_track->save();
            }
        }

        if(Auth::user()->hasRole('super-admin'))
        {
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $supplier_id;
            $account_master_track->account_master_name = "Supplier";
            $account_master_track->save();
        }else{
            $admins = User::role('super-admin')->get();
            foreach ($admins as $key => $admin) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $admin->id;
                $account_master_track->account_master_id = $supplier_id;
                $account_master_track->account_master_name = "Supplier";
                $account_master_track->save();
            }
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $supplier_id;
            $account_master_track->account_master_name = "Supplier";
            $account_master_track->save();
        }

        // if ($request->opening_balance != "") {
        //     $account = AccountMastersAccountsDetail::Where('account_master_id', $request->account_id)->first();

        //     if ($account != "") {
        //         if ($previous_opening_balance > $current_opening_balance) {
        //             $diff = $previous_opening_balance - $current_opening_balance;
        //             $account->credit =  (float)$account->credit - (float)$diff;
        //         }
        //         elseif ($previous_opening_balance < $current_opening_balance) {
        //             $diff = $current_opening_balance - $previous_opening_balance;
        //             $account->credit = (float)$account->credit + (float)$diff;
        //         }

        //         $account->save();

        //     }else{
        //         $account = new AccountMastersAccountsDetail();
        //         $account->account_master_id = $supplier->id;
        //         $account->debit = 0;
        //         $account->credit = $request->opening_balance;
        //         $account->save();
        //     }

        // }


        Session::flash('alert-success', 'Supplier updated successfully');
        return redirect('/masters/account-masters/suppliers');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Supplier  $supplier
     * @return \Illuminate\Http\Response
     */
    public function destroy($supplier_id)
    {
        $supplier = AccountMaster::Where('id', $supplier_id)->Where('account_master_type', "Supplier")->first();
        $supplier->delete();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $supplier_id)
                                                    ->Where('account_master_name', "Supplier")->get();
        foreach ($account_master_track as $item) {
            $item->delete();
        }

        Session::flash('alert-success', 'Supplier deleted successfully');
        return redirect('/masters/account-masters/suppliers');
    }

    public function softdelete($supplier_id)
    {
        $supplier = AccountMaster::Where('id', $supplier_id)->Where('account_master_type', "Supplier")->first();
        $supplier->forceDelete();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $supplier_id)
                                                    ->Where('account_master_name', "Supplier")->get();
        foreach ($account_master_track as $item) {
            $item->forceDelete();
        }

        Session::flash('alert-success', 'Supplier deleted successfully');
        return redirect('/masters/account-masters/suppliers');
    }
}
