<?php

namespace App\Http\Controllers\Report;

use App\Currency;
use App\Http\Controllers\Controller;
use App\SaleInvoice;
use App\SalesReturnVoucher;
use App\Services\Utils;
use Illuminate\Http\Request;
use PDF;
use Auth;

class SaleReportController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('sales-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();
        $from_date = $request->from_date;
        $to_date = $request->to_date;

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $sale_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        $credit_gross_total = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $credit_discount = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $credit = $credit_gross_total - $credit_discount;
        $credit_decimal = number_format($credit, $decimal, '.','');
        foreach ($sale_invoices as $key => $sale_invoice) {
            $sale_invoice-> invoice_type = 'sales';
        }

        $sale_return = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        $debit_gross_total = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $debit_discount = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $debit = $debit_gross_total - $debit_discount;
        $debit_decimal = number_format($debit, $decimal, '.', '');
        foreach ($sale_return as $key => $_sale_return) {
            $_sale_return-> invoice_type = 'sale return';
        }

        $unsorted_data  = $sale_invoices->concat($sale_return);

        $collection = collect($unsorted_data);
        $sorted_data = $collection->sortBy('date')->values()->all();

        $total = $credit - $debit;
        $total_decimal = number_format($total, $decimal, '.', '');


        return view('user.reports.sales_report.index', compact('decimal','sorted_data','from_date','to_date', 'credit_decimal', 'debit_decimal', 'total_decimal'));
    }

    public function display()
    {
        $user = Auth::user();
        if (!$user->can('sales-report')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.reports.sales_report.select_date');
    }

    public function pdfView($from_date, $to_date){

        $user = Auth::user();
        if (!$user->can('sales-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();


        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $sale_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        $credit_gross_total = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $credit_discount = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $credit = $credit_gross_total - $credit_discount;
        $credit_decimal = number_format($credit, $decimal, '.','');
        foreach ($sale_invoices as $key => $sale_invoice) {
            $sale_invoice-> invoice_type = 'sales';
        }

        $sale_return = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        $debit_gross_total = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $debit_discount = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $debit = $debit_gross_total - $debit_discount;
        $debit_decimal = number_format($debit, $decimal, '.', '');
        foreach ($sale_return as $key => $_sale_return) {
            $_sale_return-> invoice_type = 'sale return';
        }

        $unsorted_data  = $sale_invoices->concat($sale_return);

        $collection = collect($unsorted_data);
        $sorted_data = $collection->sortBy('date')->values()->all();

        $total = $credit - $debit;
        $total_decimal = number_format($total, $decimal, '.', '');

        $pdf = PDF::loadView('user.reports.sales_report.list', compact('sorted_data', 'total_decimal', 'credit_decimal', 'debit_decimal', 'from_date', 'to_date', 'decimal'));
        return $pdf->stream();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
