<?php

namespace App\Http\Controllers\Transaction;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;


use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
use App\Services\Slug;
use App\WareHouse;
use App\ItemName;
use App\PurchaseInvoice;
use App\PurchaseInvoiceItem;
use App\StaffAndSalesman;
use App\Supplier;
use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use App\AccountMasterTrack;
use App\TermsAndConditionsPurchaseInvoice;
use App\Company;
use App\Currency;
use App\WareHouseTrack;
use App\PurchaseSalesTransferStockTrack;
use App\SaleInvoiceItem;

class PurchaseInvoiceController extends Controller
{
    protected $utils;
    protected $slug;

    public function __construct(Utils $utils, Slug $slug)
    {
        $this->utils = $utils;
        $this->slug = $slug;
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('purchase-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $purchase_invoices = PurchaseInvoice::withTrashed()->with('account','items')->where('location_id', $location_id)->Where('invoice_no', '!=', 0)->orderBy('created_at','DESC')->paginate(100);
        $sales_items = [];
        foreach ($purchase_invoices as $invoice) {
            foreach ($invoice->items as $item) {
                $sales_item = SaleInvoiceItem::where('imei_serial_no', $item->imei_serial_no)->first();
                if($sales_item){
                    $sales_items[] = $item->purchase_invoice_id;
                }
            }
        }
        $sales_items = array_unique($sales_items);
        // dd($sales_items);
        // $sales_invoice_items = SaleInvoiceItem::Where('location_id', $location_id)->get();
        return view('user.transactions.purchase_invoices.index', compact('purchase_invoices','sales_items'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('purchase-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $latest_invoice_vat = PurchaseInvoice::where('location_id', $location_id)->Where('vat_check', 1)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
        if($latest_invoice_vat){
            $invoice_no_vat = $latest_invoice_vat-> invoice_no + 1;
        }else{
            $invoice_no_vat = 1;
        }

        $latest_invoice = PurchaseInvoice::where('location_id', $location_id)->Where('vat_check', 0)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
        if($latest_invoice){
            $invoice_no = (int)$latest_invoice-> invoice_no + 1 .'A' ;

        }else{
            $invoice_no = '1A';
        }

        $salesmen = StaffAndSalesman::where('job_designation', 'salesman')->get();

        $tax_accounts = AccountMaster::where('class_of_account_id', 18)->get();

        $warehouses = WareHouseTrack::with(['warehouse' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $item_names = ItemName::Where('location_id', $location_id)->get();
        $currency = Currency::Where('location_id', $location_id)->first();
        $terms_and_conditions_purchase_invoice = TermsAndConditionsPurchaseInvoice::where('location_id', $location_id)->first();
        return view('user.transactions.purchase_invoices.create', compact('currency','invoice_no_vat','invoice_no', 'warehouses', 'salesmen', 'tax_accounts', 'account_masters','item_names', 'terms_and_conditions_purchase_invoice'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request->all());
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $purchase_invoice = new PurchaseInvoice();
            $purchase_invoice->location_id = $location_id;
            $purchase_invoice->invoice_no = $request->invoice_no;
            $purchase_invoice->mode_of_payment = $request->mode_of_payment;
            $purchase_invoice->date = $request->date;
            $purchase_invoice->purchase_order = $request->purchase_order;
            $purchase_invoice->lpo_days = $request->lpo_days;
            if ($request->vat_check == "") {
                $purchase_invoice->vat_check = 0;
            }else {
                $purchase_invoice->vat_check = $request->vat_check;
            }
            // $purchase_invoice->warehouse_id = $request->warehouse_id;
            $purchase_invoice->account_id = $request->account_id;
            $purchase_invoice->total_quantity = $request->total_quantity;
            $purchase_invoice->gross_total = $request->gross_total;
            $purchase_invoice->discount = $request->discount_amount;
            $purchase_invoice->tax_account_id= $request->tax_account_id;
            $purchase_invoice->vat_percentage = $request->vat_percentage;
            $purchase_invoice->vat_amount = $request->vat_amount;
            $purchase_invoice->net_total = $request->net_total;
            $purchase_invoice->paid_amount = 0;
            $purchase_invoice->remaining_amount = $request->net_total;
            $purchase_invoice->amount_in_words = $request->amount_in_words;
            $purchase_invoice->narrations = $request->narrations;
            $purchase_invoice->buyer_name = $request->buyer_name;
            $purchase_invoice->buyer_mobile = $request->buyer_mobile;
            $purchase_invoice->buyer_email = $request->buyer_email;
            $purchase_invoice->buyer_address = $request->buyer_address;
            $purchase_invoice->buyer_vat_number = $request->buyer_vat_number;
            $purchase_invoice->is_signed = 0;
            $purchase_invoice->save();

            foreach ($request->item_name_id as $key => $item_name_id) {
                $purchase_invoice_item = new PurchaseInvoiceItem();
                $purchase_invoice_item-> purchase_invoice_id = $purchase_invoice-> id;
                $purchase_invoice_item-> item_name_id = $request->item_name_id[$key];
                $purchase_invoice_item-> item_unit_id = $request->item_unit[$key];
                $purchase_invoice_item-> warehouse_id = $request->warehouse_id[$key];
                $purchase_invoice_item-> description = $request->description[$key];
                // $purchase_invoice_item-> color = $request->color[$key];
                $purchase_invoice_item-> imei_serial_no = $request->imei_serial_no[$key];
                $purchase_invoice_item-> quantity = $request->quantity[$key];
                $purchase_invoice_item-> rate_per_quantity = $request->rate_per_quantity[$key];
                $purchase_invoice_item-> amount = $request->amount[$key];
                $purchase_invoice_item-> remarks = $request->remarks[$key];
                $purchase_invoice_item-> save();

                $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $request->imei_serial_no[$key])->Where('location_id', $location_id)->first();
                if ($purchase_sales_transfer_stock_track) {
                    return redirect()->back()->withErrors("IMEI Already Exist In This Location");
                    // $purchase_sales_transfer_stock_track-> location_id = $location_id;
                    // $purchase_sales_transfer_stock_track-> warehouse_id = $request->warehouse_id[$key];
                    // $purchase_sales_transfer_stock_track-> item_name_id = $request->item_name_id[$key];
                    // $purchase_sales_transfer_stock_track-> item_unit_id = $request->item_unit[$key];
                    // $purchase_sales_transfer_stock_track-> description = $request->description[$key];
                    // $purchase_sales_transfer_stock_track-> color = $request->color[$key];
                    // $purchase_sales_transfer_stock_track-> quantity = $purchase_sales_transfer_stock_track->quantity + $request->quantity[$key];
                    // $purchase_sales_transfer_stock_track-> rate_per_quantity = $request->rate_per_quantity[$key];
                    // $purchase_sales_transfer_stock_track-> amount = $request->amount[$key];
                    // $purchase_sales_transfer_stock_track->save();
                }else {
                    $purchase_sales_transfer_stock_track = new PurchaseSalesTransferStockTrack();
                    $purchase_sales_transfer_stock_track-> location_id = $location_id;
                    $purchase_sales_transfer_stock_track-> warehouse_id = $request->warehouse_id[$key];
                    $purchase_sales_transfer_stock_track-> item_name_id = $request->item_name_id[$key];
                    $purchase_sales_transfer_stock_track-> item_unit_id = $request->item_unit[$key];
                    $purchase_sales_transfer_stock_track-> description = $request->description[$key];
                    // $purchase_sales_transfer_stock_track-> color = $request->color[$key];
                    $purchase_sales_transfer_stock_track-> imei_serial_no = $request->imei_serial_no[$key];
                    $purchase_sales_transfer_stock_track-> quantity = $request->quantity[$key];
                    $purchase_sales_transfer_stock_track-> rate_per_quantity = $request->rate_per_quantity[$key];
                    $purchase_sales_transfer_stock_track-> amount = $request->amount[$key];
                    $purchase_sales_transfer_stock_track-> save();
                }
            }

            // $account = AccountMastersAccountsDetail::Where('account_master_id', $request->account_id)->first();

            // if ($account == "") {
            //     $account = new AccountMastersAccountsDetail;
            //     $account->account_master_id = $request->account_id;
            //     $account->debit = 0;
            //     $account->credit = $request->net_total;
            //     $account->save();
            // }
            // elseif ($account != "") {
            //     $credit = $account->credit;

            //     $account->credit = (float)$credit + (float)$request->net_total;

            //     $account->save();
            // }

            //for accounts
            $account = new AccountMastersAccountsDetail();
            $account->account_master_id = $request->account_id;
            $account->debit = 0;
            $account->credit = $request->net_total;
            $account->date = $request->date;
            $account->voucher_type = "purchase";
            $account->voucher_no = $request->invoice_no;
            $account->save();

            //for tax account
            $tax_account = new AccountMastersAccountsDetail();
            $tax_account->account_master_id = $request->tax_account_id;
            $tax_account->credit = 0;
            $tax_account->debit = $request->vat_amount;
            $tax_account->date = $request->date;
            $tax_account->voucher_type = "purchase";
            $tax_account->voucher_no = $request->invoice_no;
            $tax_account->save();

            DB::commit();
            Session::flash('alert-success','Purchase Invoice created successfully');
            return redirect('/transactions/purchase-invoices');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\PurchaseInvoice  $purchaseInvoice
     * @return \Illuminate\Http\Response
     */
    public function show(PurchaseInvoice $purchaseInvoice)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\PurchaseInvoice  $purchaseInvoice
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = Auth::user();
        if (!$user->can('edit-purchase-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $purchase_invoice = PurchaseInvoice::with('items.name','items.unit')->where('id', $id)->first();

        if ($purchase_invoice->vat_check == 1) {

            $invoice_no_vat = $purchase_invoice->invoice_no;

            $latest_invoice = PurchaseInvoice::where('location_id', $location_id)->Where('vat_check', 0)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
            if($latest_invoice){
                $invoice_no = (int)$latest_invoice-> invoice_no + 1 .'A' ;

            }else{
                $invoice_no = '1A';
            }

        }elseif ($purchase_invoice->vat_check == 0) {

            $invoice_no = $purchase_invoice->invoice_no;

            $latest_invoice_vat = PurchaseInvoice::where('location_id', $location_id)->Where('vat_check', 1)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
            if($latest_invoice_vat){
                $invoice_no_vat = $latest_invoice_vat-> invoice_no + 1;
            }else{
                $invoice_no_vat = 1;
            }
        }

        $company = Company::where('id', 1)->first();
        // return $company;

        $salesmen = StaffAndSalesman::where('job_designation', 'salesman')->get();

        $tax_accounts = AccountMaster::where('class_of_account_id', 18)->get();

        $warehouses = WareHouseTrack::with(['warehouse' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $account_id = $purchase_invoice->account_id;
        $account = AccountMaster::Find($account_id);

        if ($account->transactions_password != "") {
            $pass_check = "yes";
        }elseif ($account->transactions_password == "") {
            $pass_check = "no";
        }

        $item_names = ItemName::Where('location_id', $location_id)->get();
        $currency = Currency::Where('location_id', $location_id)->first();
        $terms_and_conditions_purchase_invoice = TermsAndConditionsPurchaseInvoice::where('location_id', $location_id)->first();
        return view('user.transactions.purchase_invoices.edit', compact('invoice_no','invoice_no_vat','currency','purchase_invoice', 'company', 'warehouses', 'salesmen', 'tax_accounts', 'account_masters','item_names', 'terms_and_conditions_purchase_invoice', 'pass_check', 'account'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\PurchaseInvoice  $purchaseInvoice
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        // dd($request->all());
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $purchase_invoice = PurchaseInvoice::where('id', $id)->first();
            // dd($purchase_invoice->net_total);
            $previous_net_total = (float)$purchase_invoice->net_total;
            $current_net_total = (float)$request->net_total;
            $account = AccountMastersAccountsDetail::Where('account_master_id', $purchase_invoice->account_id)->Where('voucher_type', "purchase")->Where('voucher_no', $purchase_invoice->invoice_no)->first();
            $purchase_invoice->location_id = $location_id;
            $purchase_invoice->invoice_no = $request->invoice_no;
            $purchase_invoice->mode_of_payment = $request->mode_of_payment;
            $purchase_invoice->date = $request->date;
            $purchase_invoice->purchase_order = $request->purchase_order;
            $purchase_invoice->lpo_days = $request->lpo_days;
            if ($request->vat_check == "") {
                $purchase_invoice->vat_check = 0;
            }else {
                $purchase_invoice->vat_check = $request->vat_check;
            }
            // $purchase_invoice->warehouse_id = $request->warehouse_id;
            $purchase_invoice->account_id = $request->account_id;
            $purchase_invoice->total_quantity = $request->total_quantity;
            $purchase_invoice->gross_total = $request->gross_total;
            $purchase_invoice->discount = $request->discount_amount;
            $purchase_invoice->tax_account_id= $request->tax_account_id;
            $purchase_invoice->vat_percentage = $request->vat_percentage;
            $purchase_invoice->vat_amount = $request->vat_amount;
            $purchase_invoice->remaining_amount = $request->net_total;
            $purchase_invoice->net_total = $request->net_total;
            $purchase_invoice->paid_amount = 0;
            $purchase_invoice->remaining_amount = $request->net_total;
            $purchase_invoice->amount_in_words = $request->amount_in_words;
            $purchase_invoice->narrations = $request->narrations;
            $purchase_invoice->buyer_name = $request->buyer_name;
            $purchase_invoice->buyer_mobile = $request->buyer_mobile;
            $purchase_invoice->buyer_email = $request->buyer_email;
            $purchase_invoice->buyer_address = $request->buyer_address;
            $purchase_invoice->buyer_vat_number = $request->buyer_vat_number;
            $purchase_invoice->save();


            // $purchase_items = PurchaseInvoiceItem::with('purchase_sale_stock')->where('purchase_invoice_id', $id)->get();
            $purchase_items = PurchaseInvoiceItem::with(['purchase_sale_stock' => function($q) use ($location_id) {
                $q->where('location_id', $location_id);}])->where('purchase_invoice_id', $id)->get();

            foreach ($purchase_items as $item) {
                $item->purchase_sale_stock()->forcedelete();
                $item->forceDelete();
                // $purchase_sale_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $item->imei_serial_no)->first();
                // $quantity = $item->quantity;
                // $purchase_sale_track->quantity = $purchase_sale_track->quantity - $quantity;
                // $purchase_sale_track->save();

                // $item->forcedelete();
            }

            // PurchaseInvoiceItem::where('purchase_invoice_id', $id)->purchase_sale_stock->forcedelete();

            foreach ($request->item_name_id as $key => $item_name_id) {
                $purchase_invoice_item = new PurchaseInvoiceItem();
                $purchase_invoice_item-> purchase_invoice_id = $purchase_invoice-> id;
                $purchase_invoice_item-> item_name_id = $request->item_name_id[$key];
                $purchase_invoice_item-> item_unit_id = $request->item_unit[$key];
                $purchase_invoice_item-> warehouse_id = $request->warehouse_id[$key];
                $purchase_invoice_item-> description = $request->description[$key];
                // $purchase_invoice_item-> color = $request->color[$key];
                $purchase_invoice_item-> imei_serial_no = $request->imei_serial_no[$key];
                $purchase_invoice_item-> quantity = $request->quantity[$key];
                $purchase_invoice_item-> rate_per_quantity = $request->rate_per_quantity[$key];
                $purchase_invoice_item-> amount = $request->amount[$key];
                $purchase_invoice_item-> remarks = $request->remarks[$key];
                $purchase_invoice_item-> save();

                $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $request->imei_serial_no[$key])->Where('location_id', $location_id)->first();
                if ($purchase_sales_transfer_stock_track) {
                    return redirect()->back()->withErrors("IMEI Already Exist In This Location");
                    // $purchase_sales_transfer_stock_track-> location_id = $location_id;
                    // $purchase_sales_transfer_stock_track-> warehouse_id = $request->warehouse_id[$key];
                    // $purchase_sales_transfer_stock_track-> item_name_id = $request->item_name_id[$key];
                    // $purchase_sales_transfer_stock_track-> item_unit_id = $request->item_unit[$key];
                    // $purchase_sales_transfer_stock_track-> description = $request->description[$key];
                    // $purchase_sales_transfer_stock_track-> color = $request->color[$key];
                    // $purchase_sales_transfer_stock_track-> quantity = $purchase_sales_transfer_stock_track->quantity + $request->quantity[$key];
                    // $purchase_sales_transfer_stock_track-> rate_per_quantity = $request->rate_per_quantity[$key];
                    // $purchase_sales_transfer_stock_track-> amount = $request->amount[$key];
                    // $purchase_sales_transfer_stock_track->save();
                }else {
                    $purchase_sales_transfer_stock_track = new PurchaseSalesTransferStockTrack();
                    $purchase_sales_transfer_stock_track-> location_id = $location_id;
                    $purchase_sales_transfer_stock_track-> warehouse_id = $request->warehouse_id[$key];
                    $purchase_sales_transfer_stock_track-> item_name_id = $request->item_name_id[$key];
                    $purchase_sales_transfer_stock_track-> item_unit_id = $request->item_unit[$key];
                    $purchase_sales_transfer_stock_track-> description = $request->description[$key];
                    // $purchase_sales_transfer_stock_track-> color = $request->color[$key];
                    $purchase_sales_transfer_stock_track-> imei_serial_no = $request->imei_serial_no[$key];
                    $purchase_sales_transfer_stock_track-> quantity = $request->quantity[$key];
                    $purchase_sales_transfer_stock_track-> rate_per_quantity = $request->rate_per_quantity[$key];
                    $purchase_sales_transfer_stock_track-> amount = $request->amount[$key];
                    $purchase_sales_transfer_stock_track-> save();
                }
            }

            // $account = AccountMastersAccountsDetail::Where('account_master_id', $request->account_id)->Where('voucher_type', "purchase")->Where('voucher_no', $purchase_invoice->invoice_no)->first();
            $account->account_master_id = $request->account_id;
            if ($previous_net_total > $current_net_total) {
                $diff = $previous_net_total - $current_net_total;
                $diff_calculate =  (float)$account->credit - (float)$diff;
                $account->credit =  number_format((float)$diff_calculate, $request->currency_decimal, '.', '');
            }
            elseif ($previous_net_total < $current_net_total) {
                $diff = $current_net_total - $previous_net_total;
                $diff_calculate = (float)$account->credit + (float)$diff;
                $account->credit = number_format((float)$diff_calculate, $request->currency_decimal, '.', '');
            }
            $account-> voucher_no = $request->invoice_no;

            $account->save();

            //for tax account
            $tax_account = AccountMastersAccountsDetail::Where('account_master_id', $request->tax_account_id)->Where('voucher_type', "purchase")->Where('voucher_no', $purchase_invoice->invoice_no)->first();
            $tax_account->account_master_id = $request->tax_account_id;
            $tax_account->credit = 0;
            $tax_account->debit = $request->vat_amount;
            $tax_account->date = $request->date;
            $tax_account->voucher_type = "purchase";
            $tax_account->voucher_no = $request->invoice_no;
            $tax_account->save();

            DB::commit();
            Session::flash('alert-success','Purchase Invoice Updated successfully');
            return redirect('/transactions/purchase-invoices');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\PurchaseInvoice  $purchaseInvoice
     * @return \Illuminate\Http\Response
     */
    public function destroy(PurchaseInvoice $purchaseInvoice)
    {
        //
    }

    public function view($id)
    {
        $user = Auth::user();
        if (!$user->can('view-print-purchase-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $purchase_invoice = PurchaseInvoice::withTrashed()->with('items_cancelled.name')->where('id', $id)->first();
        $buyer = AccountMaster::Where('id', $purchase_invoice->account_id)->first();
        $company = Company::where('location_id', $location_id)->first();
        // return $company;
        $warehouses = WareHouse::all();
        $salesmen = StaffAndSalesman::where('job_designation', 'salesman')->get();

        $account_masters = [];
        $account_master_tracks = AccountMasterTrack::Where('user_id', auth()->user()->id)->get();
        foreach ($account_master_tracks as $key => $account_master_track){
            $account_masters[] = AccountMaster::Where('location_id', $location_id)->Where('id', $account_master_track->account_master_id)->first();
        }

        $item_names = ItemName::all();
        $currency = Currency::Where('location_id', $location_id)->first();
        $terms_and_conditions_purchase_invoice = TermsAndConditionsPurchaseInvoice::where('location_id', $location_id)->first();
        $tax_account = AccountMaster::where('id', $purchase_invoice->tax_account_id)->first();
        return view('user.print.purchase_invoice.index', compact('buyer','currency', 'purchase_invoice', 'company', 'warehouses', 'salesmen', 'tax_account', 'account_masters','item_names', 'terms_and_conditions_purchase_invoice'));
    }

    public function softdelete($id)
    {
        $user = Auth::user();
        if (!$user->can('cancel-purchase-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $purchase_invoice = PurchaseInvoice::Find($id);
        $net_total = $purchase_invoice->net_total;
        $account_id = $purchase_invoice->account_id;

        $account = AccountMastersAccountsDetail::Where('account_master_id', $account_id)->Where('voucher_type', "purchase")->Where('voucher_no', $purchase_invoice->invoice_no)->first();
        // $credit = $account->credit;
        // $account->credit = (float)$credit - (float)$net_total;
        $account->delete();

        $purchase_invoice->delete();

        $purchase_invoice_items = PurchaseInvoiceItem::Where('purchase_invoice_id', $id)->get();
        foreach ($purchase_invoice_items as $item) {
            $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $item->imei_serial_no)->first();
            if ($purchase_sales_transfer_stock_track) {
                $purchase_sales_transfer_stock_track->quantity = $purchase_sales_transfer_stock_track->quantity - $item->quantity;
                $purchase_sales_transfer_stock_track->imei_serial_no = $purchase_sales_transfer_stock_track->imei_serial_no . '-cancelled';
                $purchase_sales_transfer_stock_track->save();
            }else{
                return redirect()->back()->withErrors("IMEI Not Found");
            }

            $item->imei_serial_no = $item->imei_serial_no . '-cancelled';
            $item-> save();
            $item->delete();
        }
        Session::flash('alert-success','Purchase Invoice Cancelled successfully');
        return redirect('/transactions/purchase-invoices');
    }

    public function signature($id){
        $user = Auth::user();
        if (!$user->can('purchase-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $purchase_invoice = PurchaseInvoice::Find($id);
        return view('user.transactions.purchase_invoices.signature', compact('purchase_invoice'));
    }

    public function signatureUpload(Request $request, $id){
        $purchase_invoice = PurchaseInvoice::Find($id);
        $folderPath = public_path('/user_files/img/client_signs/');
        $image_parts = explode(";base64,", $request->signed);
        $image_type_aux = explode("image/", $image_parts[0]);
        $image_type = $image_type_aux[1];
        $image_base64 = base64_decode($image_parts[1]);
        $file_name = 'purchaseinvoice-' . $id . '.'.$image_type;
        $file = $folderPath . $file_name;
        file_put_contents($file, $image_base64);

        $purchase_invoice->is_signed = 1;
        $purchase_invoice->signature_path = $file_name;
        $purchase_invoice->save();

        Session::flash('alert-success','Purchase Invoice Signature Uploaded successfully');
        return redirect('/transactions/purchase-invoices');
    }
}
