<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
use App\Currency;

class CurrencyController extends Controller
{

    protected $utils;

    public function __construct(Utils $utils)
    {
        $this->utils = $utils;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('currency-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $currencies = Currency::Where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(25);
        return view('user.masters.currencies.index', compact('currencies'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('currency-master')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.masters.currencies.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $location_id = $this->getLocationId();
        $currency = new Currency();
        $currency->location_id = $location_id;
        $currency->currency_id = $request->currency_id;
        $currency->currency_detail = $request->currency_detail;
        $currency->decimal_currency = $request->decimal_currency;
        $currency->minor_unit = $request->minor_unit;
        $currency->save();
        Session::flash('alert-success', 'Currency created successfully');
        return redirect('/masters/currencies');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Currency  $currency
     * @return \Illuminate\Http\Response
     */
    public function show(Currency $currency)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Currency  $currency
     * @return \Illuminate\Http\Response
     */
    public function edit($currency_id)
    {
        $user = Auth::user();
        if (!$user->can('currency-master')) {
            abort(403, 'Unauthorized action.');
        }

        $currency = Currency::find($currency_id);
        return view('user.masters.currencies.edit', compact('currency'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Currency  $currency
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $currency_id)
    {

        $currency = Currency::find($currency_id);
        $currency->currency_id = $request->currency_id;
        $currency->currency_detail = $request->currency_detail;
        $currency->decimal_currency = $request->decimal_currency;
        $currency->minor_unit = $request->minor_unit;
        $currency->save();
        Session::flash('alert-success', 'Currency updated successfully');
        return redirect('/masters/currencies');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Currency  $currency
     * @return \Illuminate\Http\Response
     */
    public function destroy($currency_id)
    {
        $currency = Currency::findOrFail($currency_id);
        $currency->delete();
        Session::flash('alert-success', 'Currency deleted successfully');
        return redirect('/masters/currencies');
    }
}
