<?php

namespace App\Http\Controllers;

use App\Branches;
use App\ClassOfAccount;
use App\Country;
use App\Supplier;
use App\Http\Requests\StoreSupplierRequest;
use App\Http\Requests\UpdateSupplierRequest;
use Illuminate\Support\Facades\Storage;

class SupplierMasterController extends Controller
{
    public function index()
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))->orderBy('id', 'desc')->get();
        return view('crusher.supplier_master.index', compact('suppliers'));
    }

    public function create()
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);

        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        // $maintainAccount = Maintain::all();
        return view('crusher.supplier_master.create', compact('countries', 'classOfAccount'));
    }

    public function store(StoreSupplierRequest $request)
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        try {

            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');

            // Handle file uploads for documents
            for ($i = 1; $i <= 7; $i++) {
                if ($request->hasFile("doc_file_$i")) {
                    $file = $request->file("doc_file_$i");
                    $fileName = time() . '_doc_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/supplier_master/docs/', $fileName, 'public');
                    $validatedData["doc_file_$i"] = $filePath;
                }
            }

        // Create a new supplier record
        Supplier::create($validatedData);

            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.suppliers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function edit($supplier)
    {
        if(!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) return response()->json(['status'=>false, 'error'=>'Unauthorized'], 403);
        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        $supplier = Supplier::findOrFail($supplier);
        return view('crusher.supplier_master.edit', compact('countries', 'classOfAccount', 'supplier'));
    }

    public function update(UpdateSupplierRequest $request, Supplier $supplier)
{
    if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) {
        return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
    }

    try {
        $validatedData = $request->validated();
        $validatedData['branch_id'] = session()->get('selected_branch');

        // Handle file uploads for authorized signatory signature samples
        for ($i = 1; $i <= 3; $i++) {
            if ($request->hasFile("authorised_sign_{$i}_sign_sample")) {
                $file = $request->file("authorised_sign_{$i}_sign_sample");
                $fileName = time() . '_auth_sign_' . $i . '.' . $file->getClientOriginalExtension();
                $filePath = $file->storeAs('uploads/admin/supplier_master/authorised_docs/', $fileName, 'public');
                $validatedData["authorised_sign_{$i}_sign_sample"] = $filePath;

                // Delete existing file if it exists
                if (!empty($supplier["authorised_sign_{$i}_sign_sample"])) {
                    Storage::disk('public')->delete($supplier["authorised_sign_{$i}_sign_sample"]);
                }
            }
        }

        // Handle file uploads for documents
        for ($i = 1; $i <= 7; $i++) {
            if ($request->hasFile("doc_file_$i")) {
                $file = $request->file("doc_file_$i");
                $fileName = time() . '_doc_' . $i . '.' . $file->getClientOriginalExtension();
                $filePath = $file->storeAs('uploads/admin/supplier_master/docs/', $fileName, 'public');
                $validatedData["doc_file_$i"] = $filePath;

                // Delete existing file if it exists
                if (!empty($supplier["doc_file_$i"])) {
                    Storage::disk('public')->delete($supplier["doc_file_$i"]);
                }
            }
        }

        // Update the supplier record
        $supplier->update($validatedData);

        flash()->addSuccess('Record Updated');
        return redirect()->route('crusher.suppliers.index');
    } catch (\Throwable $th) {
        flash()->addError($th->getMessage());
        return redirect()->back()->withInput();
    }
}


    public function softDelete($supplier)
    {
        $supplier = Supplier::findOrFail($supplier);
        $supplier->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.suppliers.index');
    }

    public function show($supplier)
    {
        if(!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'customers-account-master')) return response()->json(['status'=>false, 'error'=>'Unauthorized'], 403);
        $supplier = Supplier::with('country', 'classOfAccount')->findOrFail($supplier);
        return view('crusher.supplier_master.view', compact('supplier'));
    }
}
